@@ -1568,7 +1568,7 @@ gdk_drawable_impl_directfb_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
         {
           sizeof (GdkDrawableImplDirectFBClass),
           (GBaseInitFunc) NULL,
@@ -1642,7 +1642,7 @@ gdk_directfb_ref_cairo_surface (GdkDrawable *drawable)
   if (!impl->cairo_surface) {
     IDirectFBSurface *surface;
     g_assert (impl->surface != NULL);
-#if defined(CAIRO_VERSION_CODE) && CAIRO_VERSION_CODE >= CAIRO_VERSION_ENCODE(1,5,5)
+#if defined(CAIRO_VERSION) && CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1,5,5)
     impl->surface->GetSubSurface (impl->surface, NULL, &surface);
 #else
     surface = impl->surface;
@@ -1654,7 +1654,7 @@ gdk_directfb_ref_cairo_surface (GdkDrawable *drawable)
                                      &gdk_directfb_cairo_key, drawable, 
                                      gdk_directfb_cairo_surface_destroy);
       }
-#if defined(CAIRO_VERSION_CODE) && CAIRO_VERSION_CODE >= CAIRO_VERSION_ENCODE(1,5,5)
+#if defined(CAIRO_VERSION) && CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1,5,5)
       surface->Release (surface);
 #endif
     }
