@@ -21,9 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GDK_H_INSIDE__) && !defined (GDK_COMPILATION)
+#error "Only <gdk/gdk.h> can be included directly."
+#endif
+
 #ifndef __GDK_PIXMAP_H__
 #define __GDK_PIXMAP_H__
 
@@ -47,9 +51,9 @@ struct _GdkPixmapObject
 {
   GdkDrawable parent_instance;
   
-  GdkDrawable *impl;  /* window-system-specific delegate object */
+  GdkDrawable *GSEAL (impl);  /* window-system-specific delegate object */
 
-  gint depth;
+  gint GSEAL (depth);
 };
 
 struct _GdkPixmapObjectClass
@@ -115,10 +119,10 @@ GdkPixmap*    gdk_pixmap_foreign_new_for_screen  (GdkScreen       *screen,
                                                   gint             depth);
 
 #ifndef GDK_DISABLE_DEPRECATED
-#define gdk_bitmap_ref                 gdk_drawable_ref
-#define gdk_bitmap_unref               gdk_drawable_unref
-#define gdk_pixmap_ref                 gdk_drawable_ref
-#define gdk_pixmap_unref               gdk_drawable_unref
+#define gdk_bitmap_ref                 g_object_ref
+#define gdk_bitmap_unref               g_object_unref
+#define gdk_pixmap_ref                 g_object_ref
+#define gdk_pixmap_unref               g_object_unref
 #endif /* GDK_DISABLE_DEPRECATED */
 
 G_END_DECLS
