@@ -25,6 +25,8 @@
 #include "gdk.h"
 #include "gdkwindowimpl.h"
 #include "gdkprivate-quartz.h"
+#include "gdkscreen-quartz.h"
+#include "gdkinputprivate.h"
 
 static gpointer parent_class;
 
@@ -250,17 +252,20 @@ gdk_window_impl_quartz_begin_paint_region (GdkPaintable    *paintable,
 
   gdk_region_get_rectangles (clipped_and_offset_region, &rects, &n_rects);
 
+  if (n_rects == 0)
+    goto done;
+
   if (bg_pixmap == NULL)
     {
       CGContextRef cg_context;
-      gfloat r, g, b, a;
+      CGColorRef color;
       gint i;
 
       cg_context = gdk_quartz_drawable_get_context (GDK_DRAWABLE (impl), FALSE);
-      _gdk_quartz_colormap_get_rgba_from_pixel (gdk_drawable_get_colormap (window),
-                                                private->bg_color.pixel,
-                                                &r, &g, &b, &a);
-      CGContextSetRGBFillColor (cg_context, r, g, b, a);
+      color = _gdk_quartz_colormap_get_cgcolor_from_pixel (window,
+                                                           private->bg_color.pixel);
+      CGContextSetFillColorWithColor (cg_context, color);
+      CGColorRelease (color);
  
       for (i = 0; i < n_rects; i++)
         {
@@ -556,6 +561,7 @@ _gdk_quartz_window_debug_highlight (GdkWindow *window, gint number)
 {
   GdkWindowObject *private = GDK_WINDOW_OBJECT (window);
   gint x, y;
+  gint gx, gy;
   GdkWindow *toplevel;
   gint tx, ty;
   static NSWindow *debug_window[10];
@@ -584,9 +590,10 @@ _gdk_quartz_window_debug_highlight (GdkWindow *window, gint number)
   x += tx;
   y += ty;
 
-  rect = NSMakeRect (x,
-                     _gdk_quartz_window_get_inverted_screen_y (y + private->height),
-                     private->width, private->height);
+  _gdk_quartz_window_gdk_xy_to_xy (x, y + private->height,
+                                   &gx, &gy);
+
+  rect = NSMakeRect (gx, gy, private->width, private->height);
 
   if (debug_window[number] && NSEqualRects (rect, old_rect[number]))
     return;
@@ -648,15 +655,45 @@ _gdk_quartz_window_is_ancestor (GdkWindow *ancestor,
                                           gdk_window_get_parent (window)));
 }
 
-/* FIXME: It would be nice to have one function that takes an NSPoint
- * and flips the coords for any window.
- */
-gint 
-_gdk_quartz_window_get_inverted_screen_y (gint y)
-{
-  NSRect rect = [[NSScreen mainScreen] frame];
 
-  return rect.size.height - y;
+/* See notes on top of gdkscreen-quartz.c */
+void
+_gdk_quartz_window_gdk_xy_to_xy (gint  gdk_x,
+                                 gint  gdk_y,
+                                 gint *ns_x,
+                                 gint *ns_y)
+{
+  GdkScreenQuartz *screen_quartz = GDK_SCREEN_QUARTZ (_gdk_screen);
+
+  if (ns_y)
+    *ns_y = screen_quartz->height - gdk_y + screen_quartz->min_y;
+
+  if (ns_x)
+    *ns_x = gdk_x + screen_quartz->min_x;
+}
+
+void
+_gdk_quartz_window_xy_to_gdk_xy (gint  ns_x,
+                                 gint  ns_y,
+                                 gint *gdk_x,
+                                 gint *gdk_y)
+{
+  GdkScreenQuartz *screen_quartz = GDK_SCREEN_QUARTZ (_gdk_screen);
+
+  if (gdk_y)
+    *gdk_y = screen_quartz->height - ns_y + screen_quartz->min_y;
+
+  if (gdk_x)
+    *gdk_x = ns_x - screen_quartz->min_x;
+}
+
+void
+_gdk_quartz_window_nspoint_to_gdk_xy (NSPoint  point,
+                                      gint    *x,
+                                      gint    *y)
+{
+  _gdk_quartz_window_xy_to_gdk_xy (point.x, point.y,
+                                   x, y);
 }
 
 static GdkWindow *
@@ -696,7 +733,7 @@ find_child_window_helper (GdkWindow *window,
         {
           NSRect frame = NSMakeRect (0, 0, 100, 100);
           NSRect content;
-          int mask;
+          NSUInteger mask;
           int titlebar_height;
 
           mask = [child_impl->toplevel styleMask];
@@ -747,6 +784,7 @@ _gdk_quartz_window_find_child (GdkWindow *window,
   return NULL;
 }
 
+
 void
 _gdk_quartz_window_did_become_main (GdkWindow *window)
 {
@@ -789,6 +827,34 @@ _gdk_quartz_window_did_resign_main (GdkWindow *window)
   clear_toplevel_order ();
 }
 
+static NSScreen *
+get_nsscreen_for_point (gint x, gint y)
+{
+  int i;
+  NSArray *screens;
+  NSScreen *screen = NULL;
+
+  GDK_QUARTZ_ALLOC_POOL;
+
+  screens = [NSScreen screens];
+
+  for (i = 0; i < [screens count]; i++)
+    {
+      NSRect rect = [[screens objectAtIndex:i] frame];
+
+      if (x >= rect.origin.x && x <= rect.origin.x + rect.size.width &&
+          y >= rect.origin.y && y <= rect.origin.y + rect.size.height)
+        {
+          screen = [screens objectAtIndex:i];
+          break;
+        }
+    }
+
+  GDK_QUARTZ_RELEASE_POOL;
+
+  return screen;
+}
+
 void
 _gdk_window_impl_new (GdkWindow     *window,
 		      GdkWindow     *real_parent,
@@ -873,12 +939,26 @@ _gdk_window_impl_new (GdkWindow     *window,
     case GDK_WINDOW_DIALOG:
     case GDK_WINDOW_TEMP:
       {
+        NSScreen *screen;
+        NSRect screen_rect;
         NSRect content_rect;
-        int style_mask;
+        NSUInteger style_mask;
+        int nx, ny;
         const char *title;
 
-        content_rect = NSMakeRect (private->x,
-                                   _gdk_quartz_window_get_inverted_screen_y (private->y) - private->height,
+        /* initWithContentRect will place on the mainScreen by default.
+         * We want to select the screen to place on ourselves.  We need
+         * to find the screen the window will be on and correct the
+         * content_rect coordinates to be relative to that screen.
+         */
+        _gdk_quartz_window_gdk_xy_to_xy (private->x, private->y, &nx, &ny);
+
+        screen = get_nsscreen_for_point (nx, ny);
+        screen_rect = [screen frame];
+        nx -= screen_rect.origin.x;
+        ny -= screen_rect.origin.y;
+
+        content_rect = NSMakeRect (nx, ny - private->height,
                                    private->width,
                                    private->height);
 
@@ -898,7 +978,8 @@ _gdk_window_impl_new (GdkWindow     *window,
 	impl->toplevel = [[GdkQuartzWindow alloc] initWithContentRect:content_rect 
 			                                    styleMask:style_mask
 			                                      backing:NSBackingStoreBuffered
-			                                        defer:NO];
+			                                        defer:NO
+                                                                screen:screen];
 
 	if (attributes_mask & GDK_WA_TITLE)
 	  title = attributes->title;
@@ -919,6 +1000,7 @@ _gdk_window_impl_new (GdkWindow     *window,
 	impl->view = [[GdkQuartzView alloc] initWithFrame:content_rect];
 	[impl->view setGdkWindow:window];
 	[impl->toplevel setContentView:impl->view];
+	[impl->view release];
       }
       break;
 
@@ -940,6 +1022,7 @@ _gdk_window_impl_new (GdkWindow     *window,
 	    /* GdkWindows should be hidden by default */
 	    [impl->view setHidden:YES];
 	    [parent_impl->view addSubview:impl->view];
+	    [impl->view release];
 	  }
       }
       break;
@@ -954,13 +1037,61 @@ _gdk_window_impl_new (GdkWindow     *window,
     gdk_window_set_type_hint (window, attributes->type_hint);
 }
 
+void
+_gdk_quartz_window_update_position (GdkWindow *window)
+{
+  NSRect frame_rect;
+  NSRect content_rect;
+  GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+
+  GDK_QUARTZ_ALLOC_POOL;
+
+  frame_rect = [impl->toplevel frame];
+  content_rect = [impl->toplevel contentRectForFrameRect:frame_rect];
+
+  _gdk_quartz_window_xy_to_gdk_xy (content_rect.origin.x,
+                                   content_rect.origin.y + content_rect.size.height,
+                                   &private->x, &private->y);
+
+
+  GDK_QUARTZ_RELEASE_POOL;
+}
+
+void
+_gdk_windowing_update_window_sizes (GdkScreen *screen)
+{
+  GList *windows, *list;
+  GdkWindowObject *private = (GdkWindowObject *)_gdk_root;
+
+  /* The size of the root window is so that it can contain all
+   * monitors attached to this machine.  The monitors are laid out
+   * within this root window.  We calculate the size of the root window
+   * and the positions of the different monitors in gdkscreen-quartz.c.
+   *
+   * This data is updated when the monitor configuration is changed.
+   */
+  private->x = 0;
+  private->y = 0;
+  private->abs_x = 0;
+  private->abs_y = 0;
+  private->width = gdk_screen_get_width (screen);
+  private->height = gdk_screen_get_height (screen);
+
+  windows = gdk_screen_get_toplevel_windows (screen);
+
+  for (list = windows; list; list = list->next)
+    _gdk_quartz_window_update_position (list->data);
+
+  g_list_free (windows);
+}
+
 void
 _gdk_windowing_window_init (void)
 {
   GdkWindowObject *private;
   GdkWindowImplQuartz *impl;
   GdkDrawableImplQuartz *drawable_impl;
-  NSRect rect;
 
   g_assert (_gdk_root == NULL);
 
@@ -972,18 +1103,7 @@ _gdk_windowing_window_init (void)
 
   impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (_gdk_root)->impl);
 
-  /* The size of the root window should be the same as the size of
-   * the screen it belongs to.
-   *
-   * FIXME: Of course this needs to be updated when you change the monitor
-   * configuration (add another one, remove one, etc).
-   */
-  private->x = 0;
-  private->y = 0;
-  private->abs_x = 0;
-  private->abs_y = 0;
-  private->width = gdk_screen_get_width (_gdk_screen);
-  private->height = gdk_screen_get_height (_gdk_screen);
+  _gdk_windowing_update_window_sizes (_gdk_screen);
 
   private->state = 0; /* We don't want GDK_WINDOW_STATE_WITHDRAWN here */
   private->window_type = GDK_WINDOW_ROOT;
@@ -1253,10 +1373,12 @@ move_resize_window_internal (GdkWindow *window,
     {
       NSRect content_rect;
       NSRect frame_rect;
+      gint gx, gy;
 
-      content_rect =  NSMakeRect (private->x,
-                                  _gdk_quartz_window_get_inverted_screen_y (private->y + private->height),
-                                  private->width, private->height);
+      _gdk_quartz_window_gdk_xy_to_xy (private->x, private->y + private->height,
+                                       &gx, &gy);
+
+      content_rect = NSMakeRect (gx, gy, private->width, private->height);
 
       frame_rect = [impl->toplevel frameRectForContentRect:content_rect];
       [impl->toplevel setFrame:frame_rect display:YES];
@@ -1644,10 +1766,9 @@ gdk_window_quartz_get_geometry (GdkWindow *window,
        */
       if ([impl->toplevel styleMask] == NSBorderlessWindowMask)
         {
-          if (x)
-            *x = ns_rect.origin.x;
-          if (y)
-            *y = _gdk_quartz_window_get_inverted_screen_y (ns_rect.origin.y + ns_rect.size.height);
+          _gdk_quartz_window_xy_to_gdk_xy (ns_rect.origin.x,
+                                           ns_rect.origin.y + ns_rect.size.height,
+                                           x, y);
         }
       else 
         {
@@ -1720,8 +1841,12 @@ gdk_window_quartz_get_root_coords (GdkWindow *window,
 
   content_rect = [impl->toplevel contentRectForFrameRect:[impl->toplevel frame]];
 
-  tmp_x = x + content_rect.origin.x;
-  tmp_y = y + _gdk_quartz_window_get_inverted_screen_y (content_rect.origin.y + content_rect.size.height);
+  _gdk_quartz_window_xy_to_gdk_xy (content_rect.origin.x,
+                                   content_rect.origin.y + content_rect.size.height,
+                                   &tmp_x, &tmp_y);
+
+  tmp_x += x;
+  tmp_y += y;
 
   while (private != GDK_WINDOW_OBJECT (toplevel))
     {
@@ -1800,8 +1925,7 @@ gdk_window_quartz_get_pointer_helper (GdkWindow       *window,
   if (window == _gdk_root)
     {
       point = [NSEvent mouseLocation];
-      x_tmp = point.x;
-      y_tmp = _gdk_quartz_window_get_inverted_screen_y (point.y);
+      _gdk_quartz_window_nspoint_to_gdk_xy (point, &x_tmp, &y_tmp);
     }
   else
     {
@@ -1869,7 +1993,8 @@ GdkWindow *
 _gdk_windowing_window_at_pointer (GdkDisplay      *display,
 				  gint            *win_x,
 				  gint            *win_y,
-                                  GdkModifierType *mask)
+                                  GdkModifierType *mask,
+				  gboolean         get_toplevel)
 {
   GdkWindow *found_window;
   gint x, y;
@@ -1907,6 +2032,26 @@ _gdk_windowing_window_at_pointer (GdkDisplay      *display,
   if (mask)
     *mask = tmp_mask;
 
+  if (get_toplevel)
+    {
+      GdkWindowObject *w = (GdkWindowObject *)found_window;
+      /* Requested toplevel, find it. */
+      /* TODO: This can be implemented more efficient by never
+	 recursing into children in the first place */
+      if (w)
+	{
+	  /* Convert to toplevel */
+	  while (w->parent != NULL &&
+		 w->parent->window_type != GDK_WINDOW_ROOT)
+	    {
+	      *win_x += w->x;
+	      *win_y += w->y;
+	      w = w->parent;
+	    }
+	  found_window = (GdkWindow *)w;
+	}
+    }
+
   return found_window;
 }
 
@@ -2434,8 +2579,10 @@ gdk_window_get_frame_extents (GdkWindow    *window,
 
   ns_rect = [impl->toplevel frame];
 
-  rect->x = ns_rect.origin.x;
-  rect->y = _gdk_quartz_window_get_inverted_screen_y (ns_rect.origin.y + ns_rect.size.height);
+  _gdk_quartz_window_xy_to_gdk_xy (ns_rect.origin.x,
+                                   ns_rect.origin.y + ns_rect.size.height,
+                                   &rect->x, &rect->y);
+
   rect->width = ns_rect.size.width;
   rect->height = ns_rect.size.height;
 }
@@ -2445,7 +2592,7 @@ gdk_window_set_decorations (GdkWindow       *window,
 			    GdkWMDecoration  decorations)
 {
   GdkWindowImplQuartz *impl;
-  int old_mask, new_mask;
+  NSUInteger old_mask, new_mask;
   NSView *old_view;
 
   if (GDK_WINDOW_DESTROYED (window) ||
@@ -2945,4 +3092,6 @@ gdk_window_impl_iface_init (GdkWindowImplIface *iface)
   iface->queue_antiexpose = _gdk_quartz_window_queue_antiexpose;
   iface->queue_translation = _gdk_quartz_window_queue_translation;
   iface->destroy = _gdk_quartz_window_destroy;
+  iface->input_window_destroy = _gdk_input_window_destroy;
+  iface->input_window_crossing = _gdk_input_window_crossing;
 }
