@@ -173,37 +173,40 @@ lookup_cached_xatom (GdkDisplay *display,
 /**
  * gdk_x11_atom_to_xatom_for_display:
  * @display: A #GdkDisplay
- * @atom: A #GdkAtom 
- * 
+ * @atom: A #GdkAtom, or %GDK_NONE
+ *
  * Converts from a #GdkAtom to the X atom for a #GdkDisplay
- * with the same string value.
- * 
- * Return value: the X atom corresponding to @atom.
+ * with the same string value. The special value %GDK_NONE
+ * is converted to %None.
+ *
+ * Return value: the X atom corresponding to @atom, or %None
  *
  * Since: 2.2
  **/
 Atom
-gdk_x11_atom_to_xatom_for_display (GdkDisplay *display, 
-				   GdkAtom atom)
+gdk_x11_atom_to_xatom_for_display (GdkDisplay *display,
+				   GdkAtom     atom)
 {
   Atom xatom = None;
-  
+
   g_return_val_if_fail (GDK_IS_DISPLAY (display), None);
-  g_return_val_if_fail (atom != GDK_NONE, None);
+
+  if (atom == GDK_NONE)
+    return None;
 
   if (display->closed)
     return None;
-  
+
   xatom = lookup_cached_xatom (display, atom);
-  
+
   if (!xatom)
     {
       char *name;
-      
+
       g_return_val_if_fail (ATOM_TO_INDEX (atom) < virtual_atom_array->len, None);
 
       name = g_ptr_array_index (virtual_atom_array, ATOM_TO_INDEX (atom));
-      
+
       xatom = XInternAtom (GDK_DISPLAY_XDISPLAY (display), name, FALSE);
       insert_atom_pair (display, atom, xatom);
     }
@@ -293,7 +296,9 @@ gdk_x11_xatom_to_atom_for_display (GdkDisplay *display,
   GdkAtom virtual_atom = GDK_NONE;
   
   g_return_val_if_fail (GDK_IS_DISPLAY (display), GDK_NONE);
-  g_return_val_if_fail (xatom != None, GDK_NONE);
+
+  if (xatom == None)
+    return GDK_NONE;
 
   if (display->closed)
     return GDK_NONE;
@@ -543,7 +548,7 @@ gdk_property_get (GdkWindow   *window,
   Atom xtype;
   int res;
 
-  g_return_val_if_fail (!window || GDK_IS_WINDOW (window), FALSE);
+  g_return_val_if_fail (!window || GDK_WINDOW_IS_X11 (window), FALSE);
 
   if (!window)
     {
@@ -552,6 +557,8 @@ gdk_property_get (GdkWindow   *window,
       
       GDK_NOTE (MULTIHEAD, g_message ("gdk_property_get(): window is NULL\n"));
     }
+  else if (!GDK_WINDOW_IS_X11 (window))
+    return FALSE;
 
   if (GDK_WINDOW_DESTROYED (window))
     return FALSE;
@@ -677,7 +684,7 @@ gdk_property_change (GdkWindow    *window,
   Atom xproperty;
   Atom xtype;
 
-  g_return_if_fail (!window || GDK_IS_WINDOW (window));
+  g_return_if_fail (!window || GDK_WINDOW_IS_X11 (window));
 
   if (!window)
     {
@@ -688,13 +695,15 @@ gdk_property_change (GdkWindow    *window,
       
       GDK_NOTE (MULTIHEAD, g_message ("gdk_property_change(): window is NULL\n"));
     }
-
+  else if (!GDK_WINDOW_IS_X11 (window))
+    return;
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
+  gdk_window_ensure_native (window);
+
   display = gdk_drawable_get_display (window);
-  
   xproperty = gdk_x11_atom_to_xatom_for_display (display, property);
   xtype = gdk_x11_atom_to_xatom_for_display (display, type);
   xwindow = GDK_WINDOW_XID (window);
@@ -728,7 +737,7 @@ void
 gdk_property_delete (GdkWindow *window,
 		     GdkAtom    property)
 {
-  g_return_if_fail (!window || GDK_IS_WINDOW (window));
+  g_return_if_fail (!window || GDK_WINDOW_IS_X11 (window));
 
   if (!window)
     {
@@ -738,6 +747,8 @@ gdk_property_delete (GdkWindow *window,
       GDK_NOTE (MULTIHEAD, 
 		g_message ("gdk_property_delete(): window is NULL\n"));
     }
+  else if (!GDK_WINDOW_IS_X11 (window))
+    return;
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
