@@ -20,7 +20,7 @@
  *          Carlos Garnacho Parro  <carlosg@gnome.org>
  */
 
-#include <config.h>
+#include "config.h"
 #include <gdk/gdkkeysyms.h>
 #include "gtkintl.h"
 #include "gtkprivate.h"
@@ -64,7 +64,7 @@ enum {
 
 #define GTK_CELL_RENDERER_SPIN_PATH "gtk-cell-renderer-spin-path"
 
-G_DEFINE_TYPE (GtkCellRendererSpin, gtk_cell_renderer_spin, GTK_TYPE_CELL_RENDERER_TEXT);
+G_DEFINE_TYPE (GtkCellRendererSpin, gtk_cell_renderer_spin, GTK_TYPE_CELL_RENDERER_TEXT)
 
 
 static void
@@ -150,8 +150,7 @@ gtk_cell_renderer_spin_finalize (GObject *object)
   if (priv && priv->adjustment)
     g_object_unref (priv->adjustment);
 
-  if (G_OBJECT_CLASS (gtk_cell_renderer_spin_parent_class)->finalize)
-    (* G_OBJECT_CLASS (gtk_cell_renderer_spin_parent_class)->finalize) (object);
+  G_OBJECT_CLASS (gtk_cell_renderer_spin_parent_class)->finalize (object);
 }
 
 static void
@@ -208,7 +207,7 @@ gtk_cell_renderer_spin_set_property (GObject      *object,
 	}
 
       if (obj)
-	priv->adjustment = g_object_ref (obj);
+	priv->adjustment = g_object_ref_sink (obj);
       break;
     case PROP_CLIMB_RATE:
       priv->climb_rate = g_value_get_double (value);
@@ -231,7 +230,9 @@ gtk_cell_renderer_spin_focus_out_event (GtkWidget *widget,
   const gchar *new_text;
   gboolean canceled;
 
-  canceled = GTK_ENTRY (widget)->editing_canceled;
+  g_object_get (widget,
+                "editing-canceled", &canceled,
+                NULL);
 
   g_signal_handlers_disconnect_by_func (widget,
 					gtk_cell_renderer_spin_focus_out_event,
@@ -272,6 +273,21 @@ gtk_cell_renderer_spin_key_press_event (GtkWidget   *widget,
   return FALSE;
 }
 
+static gboolean
+gtk_cell_renderer_spin_button_press_event (GtkWidget      *widget,
+                                           GdkEventButton *event,
+                                           gpointer        user_data)
+{
+  /* Block 2BUTTON and 3BUTTON here, so that they won't be eaten
+   * by tree view.
+   */
+  if (event->type == GDK_2BUTTON_PRESS
+      || event->type == GDK_3BUTTON_PRESS)
+    return TRUE;
+
+  return FALSE;
+}
+
 static GtkCellEditable *
 gtk_cell_renderer_spin_start_editing (GtkCellRenderer     *cell,
 				      GdkEvent            *event,
@@ -294,9 +310,13 @@ gtk_cell_renderer_spin_start_editing (GtkCellRenderer     *cell,
   if (!priv->adjustment)
     return NULL;
 
-  spin = gtk_spin_button_new (g_object_ref (priv->adjustment),
+  spin = gtk_spin_button_new (priv->adjustment,
 			      priv->climb_rate, priv->digits);
 
+  g_signal_connect (spin, "button-press-event",
+                    G_CALLBACK (gtk_cell_renderer_spin_button_press_event),
+                    NULL);
+
   if (cell_text->text)
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin),
 			       g_ascii_strtod (cell_text->text, NULL));
@@ -304,10 +324,10 @@ gtk_cell_renderer_spin_start_editing (GtkCellRenderer     *cell,
   g_object_set_data_full (G_OBJECT (spin), GTK_CELL_RENDERER_SPIN_PATH,
 			  g_strdup (path), g_free);
 
-  g_signal_connect (G_OBJECT (spin), "focus_out_event",
+  g_signal_connect (G_OBJECT (spin), "focus-out-event",
 		    G_CALLBACK (gtk_cell_renderer_spin_focus_out_event),
 		    cell);
-  g_signal_connect (G_OBJECT (spin), "key_press_event",
+  g_signal_connect (G_OBJECT (spin), "key-press-event",
 		    G_CALLBACK (gtk_cell_renderer_spin_key_press_event),
 		    cell);
 
